/* Expression input dialog */

#include <cassert>
#include <cstdio>
#include <klineedit.h>
#include <klocale.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>

#include <expr.h>

#include "kcexprdialog.h"
#include "kcexprlineedit.h"
#include "kde.h"

/** Create an "input expression" dialog with given title and initial
    expression */
KCExprDialog::KCExprDialog (QWidget *parent, const QString &title,
			    const QString &expression)
  : KCExprDialogBase (parent)
{
  setCaption (title);
  cancelButton->setGuiItem (KStdGuiItem::cancel ());
  OKButton->setGuiItem (KStdGuiItem::ok ());
  OKButton->setEnabled (false);
  expressionText->setText (expression);
}

KCExprDialog::~KCExprDialog ()
{
}

/** Get current expression value */
QString
KCExprDialog::expression () const
{
  return expressionText->text ();
}

/* This is not thread safe! */
static KCExprDialog *error_dialog; /* = 0; */

static void
evaluate_error_handler (const QString &msg)
{
  assert (error_dialog != 0);
  /* This would under normal OO interfaces not be allowed, but
     KCExprDialogBase is public ancestor for good reasons (we want the object
     to behave reasonably), which also makes the dialog components public. */
  error_dialog->valueText->setText (i18n ("Error: %1").arg (msg));
}

/** expressionText has changed */
void
KCExprDialog::expressionChanged ()
{
  QString expr = expressionText->text ();
  if (expr.isEmpty ())
    {
      valueText->setText ("");
      OKButton->setEnabled (false);
      return;
    }

  void (*prev_error_handler) (const QString &msg) = error_handler;
  assert (error_dialog == 0);
  /* This check is not thread-safe, so it is not enough. But it should be
     enough for the unfortunate enough to try to thread this to notice
     the problem. */
  error_dialog = this;
  error_handler = evaluate_error_handler;

  intmax_t value;
  if (expr_eval (expr, &value) == 0)
    {
      char buf[64]; /* Overkill */

      assert (sizeof (value) * CHAR_BIT <= 128);
      /* Do it by hand, because Qt has too few types accepted in
	 QString::number () */
      sprintf (buf, "%jd", value);
      valueText->setText (buf);
      OKButton->setEnabled (true);
    }
  else
    OKButton->setEnabled (false);

  error_handler = prev_error_handler;
  error_dialog = 0;
}
