/* Main window of cereal_kde -*-c++-*- */
#ifndef KCMAINWIN_H__
#define KCMAINWIN_H__

#include <kparts/mainwindow.h>
#include <qptrlist.h>
#include <qworkspace.h>
#include <stdint.h>

#include "kcuistatehandler.h"

struct cereal_module;
struct me_entry;
class KActionMenu;
class KCWindow;
class QWorkspace;

/** cereal_kde main window. Note that there may be only one per process,
    because the data is stored in the C backend. */
class KCMainWin: public KParts::MainWindow, private KCUIStateHandler
{
  Q_OBJECT
  
public:
  /** Create a new window, optionally loading file (setup or state) if it is
      non-NULL */
  KCMainWin (const char *file = 0);
  ~KCMainWin ();
  /** Stupid hack... @returns true if we refuse to initialize (because the
      user selected no setup file to load) and the user should be told
      to run cereal_kwhconf */
  bool noSetupLoaded () { return noSetupLoaded_; };
  /** Register an UI state handler */
  void registerStateHandler (KCUIStateHandler *handler);
  /** Update all views, because state has changed */
  void updateViews ();
  /** Register new view */
  void registerView (KCWindow *view);
  /** Unregister view */
  void unregisterView (KCWindow *view);
  /** Select a module instance of module, ask the user if ambiguous;
      @returns selected module or NULL if none */
  struct me_entry *selectModule (const struct cereal_module *module);
  /** @returns parent for embedded views */
  QWidget *viewParent () const { return workspace; };
  /** Save current emulation state under uiState */
  void saveState (struct _xmlNode *uiState);
  /** Load current state from uiState of doc;
      returns: 0 if OK, -1 on error */
  int loadState (struct _xmlDoc *doc, struct _xmlNode *uiState);
  
protected:
  /** @return true if the window can be closed */
  bool queryClose ();

private slots:
  /** File/Open was selected */
  void fileOpen ();
  /** File/Save was selected */
  void fileSave ();
  /** File/Save as was selected */
  void fileSaveAs ();
  /** File/Quit was selected */
  void fileQuit ();
  /** Run/Step was selected */
  void runStep ();
  /** Run/Run was selected */
  void runRun ();
  /** Data/Evaluate/Modify was selected */
  void evalModify ();
  /** Data/New Watch Window was selected */
  void newWatchWindow ();
  /** Data/New Breapoint Window was selected */
  void newBreakpointWindow ();
  /** windowMenu is about to show */
  void rebuildWindowMenu ();
  /** Window/Cascade was selected */
  void windowCascade ();
  /** Window/Tile was selected */
  void windowTile ();
  /** windowMenu window entry has been selected */
  void windowSelected (int id);
  
private:
  /** @return file name we display to the user */
  QString prettyFileName ();
  /** Update window caption */
  void updateCaption ();
  /** @return true if the user has confirmed he wants to lose current state */
  bool canLoseState ();
  /** Load state or setup from file */
  void loadFile (const QString &file);
  /** Save UI state under node */
  static void saveStateCallback (struct _xmlNode *uiState, intptr_t data);
  /** Load UI state from node of doc;
      returns: 0 if OK, -1 on error */
  static int loadStateCallback (struct _xmlDoc *doc, struct _xmlNode *uiState,
				intptr_t data);

  /** true if no setup has been loaded */
  bool noSetupLoaded_;
  /** true if the state has changed */
  bool modified;
  /** Name of file this setup was loaded from / saved to, or ""  */
  QString filename;
  KActionMenu *windowMenu;
  KAction *windowCascadeAction;
  KAction *windowTileAction;
  QWorkspace *workspace;
  QPtrList<KCWindow> childViews;
  QPtrList<KCUIStateHandler> stateHandlers;
};

#endif
