/* Watch window item */

#include <cassert>
#include <cstdlib>
#include <klistview.h>
#include <qstring.h>

#include <expr.h>

#include "kcwatchitem.h"

KCWatchItem::KCWatchItem (QListView *parent, const QString &expression)
  : KListViewItem (parent), value (""), code (NULL)
{
  setExpression (expression);
}

KCWatchItem::~KCWatchItem ()
{
  free (code);
}

QString
KCWatchItem::text (int column) const
{
  switch (column)
    {
    case 0:
      return expression_;

    case 1:
      return value;

    default:
      return "";
    }
}

/** Change watch expression, compile it and compute current value */
void
KCWatchItem::setExpression (const QString &expression)
{
  struct expr_code *c;
  intmax_t val;

  c = expr_compile (expression, 0);
  if (c == NULL || expr_exec (c, &val) != 0)
    {
      free (c);
      return;
    }
  free (code);
  code = c;
  expression_ = expression;
  cerealUpdate ();
}

/** Recompute value and possibly repaint */
void
KCWatchItem::cerealUpdate ()
{
  intmax_t val;
  if (expr_eval (expression_, &val) != 0)
    return;
  QString v;
  {
    char buf[64]; /* Overkill */
    
    assert (sizeof (value) * CHAR_BIT <= 128);
    /* Do it by hand, because Qt has too few types accepted in
       QString::number () */
    sprintf (buf, "%jd", val);
    v = buf;
  }
  if (value != v)
    {
      value = v;
      repaint ();
    }
}
