/* Intel hex format handling */
#include <config.h>

#include <libxml/tree.h>

#include <lib.h>

#include "hex.h"

/* File operations for strings */
static int
string_next_char (struct file_pos *pos)
{
  if (*pos->v.string != 0)
    return (unsigned char)*pos->v.string++;
  else
    return EOF;
}

static int
string_was_error (struct file_pos *pos)
{
  (void)pos;
  return 0;
}

static const struct file_ops string_ops =
  { .next_char = string_next_char, .was_error = string_was_error };

/* Load Intel hex from XML NODE of DOC to DATA of LEN;
   returns: 0 if OK, -1 on error */
int
load_hex_xml (xmlDocPtr doc, xmlNodePtr node, uint8_t *data, size_t len)
{
  struct file_pos pos;
  xmlChar *str;
  int res;

  str = xmlNodeListGetString (doc, node->children, 1);
  if (str == NULL)
    {
      res = -1;
      goto err;
    }
  pos.filename = "HEX"; /* Dummy label */
  pos.v.string = str;
  pos.ops = &string_ops;
  res = hex_read_file (&pos, data, len);
  xmlFree (str);
 err:
  return res;
}

/* Number of bytes output per one line of save_hex_* output */
enum
{
  SAVE_HEX_PER_LINE = 16
};
